import { fromEvent } from "./event";
import { createPersistentStateStore } from "@reactive-js/async-iterator-resource";
import { concat, ofValue } from "@reactive-js/observable";
const emptyLocation = {
    fragment: "",
    path: "",
    query: "",
};
const locationEquals = (a, b) => a === b ||
    (a.path === b.path && a.query === b.query && a.fragment === b.fragment);
const chopLeadingChar = (str) => str.length > 0 ? str.substring(1) : "";
const getCurrentLocation = () => {
    const path = window.location.pathname;
    const query = chopLeadingChar(window.location.search);
    const fragment = chopLeadingChar(window.location.hash);
    return { path, query, fragment };
};
const dispatch = (newLocation) => {
    const currentLocation = getCurrentLocation();
    if (!locationEquals(currentLocation, newLocation)) {
        const { path, query, fragment } = newLocation;
        let uriString = path;
        uriString = query.length > 0 ? `${uriString}?${query}` : uriString;
        uriString = fragment.length > 0 ? `${uriString}#${fragment}` : uriString;
        window.history.pushState(undefined, "", uriString);
    }
};
const getCurrentLocationStateUpdater = (_) => {
    const uri = getCurrentLocation();
    return (_) => uri;
};
const observable = concat(ofValue(getCurrentLocationStateUpdater()), fromEvent(window, "popstate", getCurrentLocationStateUpdater));
const historyIterator = {
    dispatch,
    subscribe: subscriber => observable.subscribe(subscriber),
};
export const createLocationStoreResource = (scheduler) => createPersistentStateStore(historyIterator, emptyLocation, scheduler, locationEquals);
//# sourceMappingURL=locationStore.js.map